angular.module('beamng.apps')
.directive('engineThermalDebug', ['UiUnits', function (UiUnits) {
  return {
    templateUrl: 'modules/apps/EngineThermalDebug/app.html',
    replace: true,
    restrict: 'EA',
    scope: true,
    controller: ['$log', '$scope', 'bngApi', 'StreamsManager', function ($log, $scope, bngApi, StreamsManager) {
      var streamsList = ['engineThermalData'];
      StreamsManager.add(streamsList);
      $scope.$on('$destroy', function () {
        StreamsManager.remove(streamsList);
      });

      $scope.gearText = '';
      $scope.$on('streamsUpdate', function (event, data) {
        $scope.$evalAsync(function () {
          if (data.engineThermalData) {
            $scope.data = makeDataDisplayable(data.engineThermalData);
          } else {
            $scope.data = undefined;
          }
        });
      });

      function makeDataDisplayable (data) {
        return [{ 
              str: UiUnits.buildString('temperature', data.coolantTemperature, 0), 
              name: 'Охлаждающая жидкость', 
              warn: (data.coolantTemperature > data.thermostatTemperature && data.coolantTemperature < 120 && data.thermostatStatus == 1),
              error: (data.coolantTemperature > 120)
          }, { 
              str: UiUnits.buildString('temperature', data.oilTemperature, 0), 
              name: 'Масло', 
              warn: data.oilTemperature > 140,
              error: data.oilTemperature > 150
          }, { 
              str: UiUnits.buildString('temperature', data.engineBlockTemperature, 0), 
              name: 'Блок', 
          }, { 
              str: UiUnits.buildString('temperature', data.cylinderWallTemperature, 0), 
              name: 'Стенки целиндров', 
          }, { 
              str: UiUnits.buildString('temperature', data.exhaustTemperature, 0), 
              name: 'Выхлопной коллектор', 
          }, { 
              str: data.thermostatStatus.toFixed(3), 
              name: 'Термостат охлад жидкости', 
              warn: data.thermostatStatus > 0.9
          }, { 
              str: data.oilThermostatStatus.toFixed(3), 
              name: 'Термостат масла',
              warn: data.oilThermostatStatus > 0.9
          }, { 
              str: UiUnits.buildString('speed', data.radiatorAirSpeed, 0), 
              name: 'Скорость воздуха в радиаторе',
          }, { 
              str: data.radiatorAirSpeedEfficiency.toFixed(4), 
              name: 'Эффективность радиатора', 
          }, { 
              str: data.fanActive, 
              name: 'Вентилятор радиатора работает'
          }, { 
              str: data.coolantLeakRate.toFixed(3), 
              name: 'Скорость утечки охлад жидкости',
              warn: data.coolantLeakRate > 0,
          }, { 
              str: data.coolantEfficiency.toFixed(3), 
              name: 'Эффективность охлад жидкости',
              warn: data.coolantEfficiency < 1,
              error: data.coolantEfficiency === 0
          }, { 
              str: data.engineEfficiency.toFixed(2), 
              name: 'Эффективность двигателя', 
          }, { 
              str: UiUnits.buildString('energy', data.energyToCylinderWall, 0), 
              name: 'Q to cylinder wall', 
          }, { 
              str: UiUnits.buildString('energy', data.energyCylinderWallToCoolant, 0), 
              name: 'Q cylinder wall to coolant', 
          }, { 
              str: UiUnits.buildString('energy', data.energyCoolantToAir, 0), 
              name: 'Q coolant to air', 
          }, { 
              str: UiUnits.buildString('energy', data.energyCoolantToBlock, 0), 
              name: 'Q coolant to block', 
          }, { 
              str: UiUnits.buildString('energy', data.energyCylinderWallToBlock, 0), 
              name: 'Q cylinder wall to block', 
          }, { 
              str: UiUnits.buildString('energy', data.energyBlockToAir, 0), 
              name: 'Q block to air', 
          }, { 
              str: UiUnits.buildString('energy', data.energyToOil, 0), 
              name: 'Q to oil', 
          }, { 
              str: UiUnits.buildString('energy', data.energyCylinderWallToOil, 0), 
              name: 'Q cylinder wall to oil', 
          }, { 
              str: UiUnits.buildString('energy', data.energyOilToAir, 0), 
              name: 'Q oil radiator to air', 
          }, { 
              str: UiUnits.buildString('energy', data.energyOilSumpToAir, 0), 
              name: 'Q oil sump to air', 
          }, { 
              str: UiUnits.buildString('energy', data.energyToExhaust, 0), 
              name: 'Q to exhaust', 
          }, { 
              str: UiUnits.buildString('energy', data.energyExhaustToAir, 0), 
              name: 'Q exhaust to air', 
          }, { 
              str: data.engineBlockOverheatDamage.toFixed(), 
              name: 'Повреждения блока',
              warn: data.engineBlockOverheatDamage > 0
          }, { 
              str: data.oilOverheatDamage.toFixed(), 
              name: 'Повреждения масла',
              warn: data.oilOverheatDamage > 0
          }, { 
              str: data.cylinderWallOverheatDamage.toFixed(), 
              name: 'Повреждение стенки целиндров',
              warn: data.cylinderWallOverheatDamage > 0
          }, { 
              str: data.headGasketBlown, 
              name: 'Прокладка головки блока обдува',
              error: data.headGasketBlown
          }, { 
              str: data.pistonRingsDamaged, 
              name: 'Поршневые кольца повреждены',
              error: data.pistonRingsDamaged
          }, { 
              str: data.connectingRodBearingsDamaged, 
              name: 'Повреждены шатунные подшипники',
              error: data.connectingRodBearingsDamaged
          }
        ];
      }
    }]
  };
}]);
